Package: sem
Version: 3.1-16
Date: 2024-08-25
Title: Structural Equation Models
Authors@R: c(person("John", "Fox", role = "aut", email = "jfox@mcmaster.ca"),
	person("Zhenghua", "Nie", role = c("aut", "cre"), email="zhenghua.nie@gmail.com"),
    person("Jarrett", "Byrnes", role = "aut", email="byrnes@nceas.ucsb.edu"),
    person("Michael", "Culbertson", role = "ctb"),
    person("Saikat", "DebRoy", role = "ctb"),
    person("Michael", "Friendly", role = "ctb"),
    person("Benjamin", "Goodrich", role = "ctb"),
    person(given="Richard H.", family="Jones", role = "ctb"),
    person("Adam", "Kramer", role = "ctb"),
    person("Georges", "Monette", role = "ctb"),
    person("Frederick", "Novomestky", role = "ctb"),
    person("R-Core", role = "ctb")
	)
Depends: R (>= 3.5.0), stats
Imports: MASS, boot, mi (>= 0.9-99), utils
Suggests: polycor, DiagrammeR (>= 1.0.9)
LazyLoad: yes
LazyData: yes
ByteCompile: yes
Description: Functions for fitting general linear structural
    equation models (with observed and latent variables) using the RAM approach, 
    and for fitting structural equations in observed-variable models by two-stage least squares.
License: GPL (>= 2)
URL: https://CRAN.R-project.org/package=sem
NeedsCompilation: yes
Packaged: 2024-08-28 01:40:20 UTC; niez
Author: John Fox [aut],
  Zhenghua Nie [aut, cre],
  Jarrett Byrnes [aut],
  Michael Culbertson [ctb],
  Saikat DebRoy [ctb],
  Michael Friendly [ctb],
  Benjamin Goodrich [ctb],
  Richard H. Jones [ctb],
  Adam Kramer [ctb],
  Georges Monette [ctb],
  Frederick Novomestky [ctb],
  R-Core [ctb]
Maintainer: Zhenghua Nie <zhenghua.nie@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-28 08:10:04 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 16:09:36 UTC; unix
Archs: sem.so.dSYM
